"
 COPYRIGHT (c) 1997 by eXept Software AG / Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"


SelectionInListView subclass:#SelectionInTreeView
	instanceVariableNames:'doubleClickSelectionBlock figuresWidth leftIndent
		figuresWidthDiv2 figuresInset showLines listOfNodes imageOpened
		imageClosed imageItem showDirectoryIndicator'
	classVariableNames:'ImageOpened ImageClosed ImageItem'
	poolDictionaries:''
	category:'Views-Text'
!

!SelectionInTreeView class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1997 by eXept Software AG / Claus Gittinger
              All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

!

documentation
"
    somewhat like a SelectionInListView; but specialized for hierarchical (i.e. tree-like)
    lists and adds the functions to show/hide subtrees. 
    Requires SelectionInTree as model and TreeItem (or compatible) list entries.

    This class obsoletes SelectionInTreeView, which is no longer
    maintained but kept for backward compatibility.


    [see also:]
        SelectionInTree
        TreeItem
        SelectionInTreeView
        SelectionInListView

    [author:]
        Claus Atzkern
"

!

examples
"
    shows the tree of smalltalk classes:
                                                                        [exBegin]
      |top model sel root|

      root  := TreeItem newAsTreeFromSmalltalkClass:Object.
      root expand.
      model := SelectionInTree new root:root.
      top := StandardSystemView new.
      top extent:300@300.

      sel := SelectionInTreeView new.
      sel model: model.
      sel action:[:nr | Transcript show:'selected:'; showCR:nr].
      top add:(ScrollableView forView:sel) in:((0.0 @ 0.0 ) corner:( 1.0 @ 1.0)).
      top open.
                                                                        [exEnd]



    same, including nil-subclasses (i.e. really all classes):
                                                                        [exBegin]
      |top model sel root|

      root  :=  TreeItem newAsTreeFromSmalltalkClass:nil.
      root expand.
      model := SelectionInTree new root:root.
      top   := StandardSystemView new.
      top extent:300@300.

      sel := SelectionInTreeView new.
      sel model: model.
      sel action:[:nr | Transcript show:'selected:'; showCR:nr].

      top add:(ScrollableView forView:sel) in:((0.0 @ 0.0 ) corner:( 1.0 @ 1.0)).
      top open.
                                                                        [exEnd]



    shows the tree of smalltalk classes; show directory indication and no
    lines.
                                                                        [exBegin]
      |top model sel root|

      root  := TreeItem newAsTreeFromSmalltalkClass:Object.
      root expand.
      model := SelectionInTree new root:root.
      top := StandardSystemView new.
      top extent:300@300.

      sel := SelectionInTreeView new.
      sel showLines:false.
      sel showDirectoryIndicator:true.

      sel model: model.
      sel action:[:nr | Transcript show:'selected:'; showCR:nr].
      top add:(ScrollableView forView:sel) in:((0.0 @ 0.0 ) corner:( 1.0 @ 1.0)).
      top open.
                                                                        [exEnd]
"

! !

!SelectionInTreeView class methodsFor:'constants'!

figuresInset
    "returns horizontal right and left inset of a figure
    "
  ^ 2
!

imageClosed
    ImageClosed isNil ifTrue:[
        ImageClosed := Image fromFile:('xpmBitmaps/document_images/tiny_yellow_dir.xpm').
    ].
  ^ ImageClosed
"
ImageClosed := nil
"

!

imageItem
    ImageItem isNil ifTrue:[
        ImageItem := Image fromFile:('xpmBitmaps/document_images/tiny_file_plain.xpm')  

    ].
  ^ ImageItem
"
ImageItem := nil
"

!

imageOpened
    ImageOpened isNil ifTrue:[
        ImageOpened := Image fromFile:('xpmBitmaps/document_images/tiny_yellow_dir_open.xpm').

    ].
  ^ ImageOpened
"
ImageOpened := nil
"
! !

!SelectionInTreeView methodsFor:'accessing'!

imageClosed
    "return the value of the instance variable 'imageClosed' (automatically generated)"

    ^ imageClosed

    "Created: 3.7.1997 / 12:34:31 / cg"
!

imageClosed:something
    "set the value of the instance variable 'imageClosed' (automatically generated)"

    imageClosed := something.

    "Created: 3.7.1997 / 12:34:31 / cg"
!

imageItem
    "return the value of the instance variable 'imageItem' (automatically generated)"

    ^ imageItem

    "Created: 3.7.1997 / 12:34:34 / cg"
!

imageItem:something
    "set the value of the instance variable 'imageItem' (automatically generated)"

    imageItem := something.

    "Created: 3.7.1997 / 12:34:34 / cg"
!

imageOpened
    "return the value of the instance variable 'imageOpened' (automatically generated)"

    ^ imageOpened

    "Created: 3.7.1997 / 12:34:28 / cg"
!

imageOpened:something
    "set the value of the instance variable 'imageOpened' (automatically generated)"

    imageOpened := something.

    "Created: 3.7.1997 / 12:34:28 / cg"
!

list:aList keepSelection:keepSelection
    |list|

    list := aList.

    list size == 0 ifTrue:[
        listOfNodes := #()
    ] ifFalse:[
        (list first respondsTo:#hasChildren) ifTrue:[
            listOfNodes := aList.
            list := listOfNodes collect:[:aNode| aNode name ].
        ]
    ].
    super list:list keepSelection:keepSelection
! !

!SelectionInTreeView methodsFor:'accessing-behavior'!

doubleClickSelectionBlock
    "get the conditionBlock; this block is evaluated before a doubleClick action
     on a node containg children will be performed. In case of returning false, the
     doubleClick will not be handled.
   "
   ^ doubleClickSelectionBlock


!

doubleClickSelectionBlock:aBlock
    "set the conditionBlock; this block is evaluated before a doubleClick action
     on a node containg children will be performed. In case of returning false, the
     doubleClick will not be handled.
   "
   doubleClickSelectionBlock := aBlock


!

showDirectoryIndicator
    "returns true if directories has an open/closed indicator
    "
  ^ showDirectoryIndicator
!

showDirectoryIndicator:aState
    "set or clear open/closed indicator for directories
    "
    "show or hide lines
    "
    aState ~~ showDirectoryIndicator ifTrue:[
        showDirectoryIndicator := aState.
        self invalidate
    ].
!

showLines
    "returns true if lines are shown
    "
  ^ showLines
!

showLines:aState
    "show or hide lines
    "
    aState ~~ showLines ifTrue:[
        showLines := aState.
        self invalidate
    ].
! !

!SelectionInTreeView methodsFor:'drawing'!

drawLine:line atX:atX inVisible:visLineNr with:fg and:bg
    self drawFromVisibleLine:visLineNr to:visLineNr with:fg and:bg

!

drawLine:line fromX:x inVisible:visLineNr with:fg and:bg
    self drawFromVisibleLine:visLineNr to:visLineNr with:fg and:bg

!

drawLine:line inVisible:visLineNr with:fg and:bg
    self drawFromVisibleLine:visLineNr to:visLineNr with:fg and:bg

!

drawVisibleLine:visLineNr col:col with:fg and:bg
    self drawFromVisibleLine:visLineNr to:visLineNr with:fg and:bg

!

drawVisibleLine:visLineNr from:startCol to:endCol with:fg and:bg
    self drawFromVisibleLine:visLineNr to:visLineNr with:fg and:bg

!

drawVisibleLine:visLineNr from:startCol with:fg and:bg
    self drawFromVisibleLine:visLineNr to:visLineNr with:fg and:bg

! !

!SelectionInTreeView methodsFor:'drawing basics'!

drawFromVisibleLine:startVisLineNr to:endVisLineNr with:fg and:bg

    |nodeIndex listSize end fontHgDiv2 yBot yTop yCtr
     x x1 xFig xStr node level lvl idx image isCollapsable hasChildren radius extent xCross|

    nodeIndex := self visibleLineToAbsoluteLine:startVisLineNr.
    listSize  := listOfNodes size.
    yTop      := self yOfVisibleLine:startVisLineNr.
    end       := endVisLineNr - startVisLineNr + 1.

"/  clear rectangle line and set background color
    self paint:bg.
    self fillRectangleX:0 y:yTop width:width height:(end * fontHeight).

    nodeIndex isNil ifTrue:[^ self].
    fontHgDiv2 := fontHeight // 2.
    radius     := figuresWidthDiv2 // 2 - 1.
    extent     := radius + radius + 1.

    self paint:fg on:bg.

    (end := nodeIndex + end) > listSize ifTrue:[
        end := listSize + 1
    ].

    [nodeIndex < end] whileTrue:[
        node  := listOfNodes at:nodeIndex.
        yBot  := yTop + fontHeight.
        yCtr  := yTop + fontHgDiv2.
        image := self figureFor:node.

        (hasChildren := node hasChildren) ifTrue:[ 
            isCollapsable := node isCollapsable.
        ].

        (lvl := node level) == level ifFalse:[
            level  := lvl.
            xFig   := self xOfFigureNode:node.
            xStr   := self xOfStringNode:node.
            xCross := xFig + figuresWidthDiv2 - leftIndent.
        ].

        showLines ifTrue:[
            (isCollapsable and:[node numberOfChildren ~~ 0]) ifTrue:[
                x := xFig + figuresWidthDiv2.
                self displayLineFromX:x y:yCtr toX:x y:yBot
            ].
        "/  draw horizontal and vertical line
            self displayLineFromX:xCross y:yTop toX:xCross y:yCtr.       "/ vertical
            self displayLineFromX:xCross y:yCtr toX:xFig   y:yCtr.       "/ horizontal

        "/  draw vertical lines
            idx  := nodeIndex.

            idx ~~ listSize ifTrue:[
                x := leftIndent - figuresWidthDiv2.

                [(lvl > 0 and:[(idx := idx + 1) <= listSize])] whileTrue:[
                    node := listOfNodes at:idx.

                    node level > lvl ifFalse:[
                        x1 := (self xOfFigureNode:node) - x.
                        self displayLineFromX:x1 y:yTop toX:x1 y:yBot.
                        lvl := node level - 1
                    ]
                ]
            ]
        ].

        "/ draw image
        self displayForm:image x:xFig y:(yCtr - (image height // 2)).

        "/ draw text label
        self drawLabelIndex:nodeIndex atX:xStr y:yCtr.

        (hasChildren and:[showDirectoryIndicator]) ifTrue:[
            self drawIndicatorX:xCross y:yCtr collapsable:isCollapsable
        ].

        "/ setup next line
        nodeIndex := nodeIndex + 1.
        yTop      := yBot.
    ].






!

drawIndicatorX:xCtr y:yCtr collapsable:isCollapsable
    "draw directory indicator; the arguments received are is the center where
     to draw the indicator
    "
    |radius extent|

    radius := figuresWidthDiv2 // 2 - 1.
    extent := radius + radius + 1.

    isCollapsable ifFalse:[
        self displayRectangleX:(xCtr - radius)
                             y:(yCtr - radius)
                         width:extent
                        height:extent.
    ] ifTrue:[
        self fillRectangleX:(xCtr - radius)
                          y:(yCtr - radius)
                      width:extent
                     height:extent
    ]


!

drawLabelIndex:anIndex atX:x y:yCenter
    "draw text label at x and y centered
    "
    |y lbl|

    lbl := (listOfNodes at:anIndex) name.

    lbl notNil ifTrue:[
        y := yCenter - ((lbl heightOn:self) // 2).

        (lbl respondsTo:#string) ifTrue:[
            y := y + fontAscent.
        ].
        self displayOpaqueString:lbl x:x y:y.
    ]    
! !

!SelectionInTreeView methodsFor:'initialization'!

fetchDeviceResources
    "initialize heavily used device resources - to avoid rendering
     images again and again later"

    super fetchDeviceResources.
    self refetchDeviceResources
!

fetchImageResources
    "initialize heavily used device resources - to avoid rendering
     images again and again later; returns maximum extent of the images used.
     Could be redefined by subclass
    "
    |y x t|

    imageOpened  := (self class imageOpened) onDevice:device.
    imageClosed  := (self class imageClosed) onDevice:device.
    imageItem    := (self class imageItem)   onDevice:device.

    y := imageClosed heightOn:self.
    x := imageClosed widthOn:self.

    (t := imageOpened heightOn:self) > y ifTrue:[y := t].
    (t := imageOpened widthOn:self)  > x ifTrue:[x := t].
    (t := imageItem   heightOn:self) > y ifTrue:[y := t].
    (t := imageItem   widthOn:self)  > x ifTrue:[x := t].

  ^ x @ y

!

getFontParameters
    super getFontParameters.

    figuresWidth notNil ifTrue:[
        self refetchDeviceResources
    ]
!

initialize
    "setup instance attributes
    "
    super initialize.
    showLines := true.
    showDirectoryIndicator := false.

    figuresWidth     := 18.                             "/ default: will change during startup
    figuresInset     := 2.                              "/ default: will change during startup
    leftIndent       := figuresWidth + figuresInset.    "/ default: will change during startup
    figuresWidthDiv2 := figuresWidth // 2.              "/ default: will change during startup
!

refetchDeviceResources
    "reinitialize heavily used device resources - to avoid rendering
     images again and again later
    "
    |y extent|

    figuresInset := self class figuresInset.
    extent       := self fetchImageResources.

    extent y > fontHeight ifTrue:[
        fontHeight := extent y.
    ].

    figuresWidth     := extent x.
    leftIndent       := figuresInset + figuresWidth.
    figuresWidthDiv2 := figuresWidth // 2.

! !

!SelectionInTreeView methodsFor:'model'!

getListFromModel
    "if I have a model, get my list from it using the listMessage.
     If listMessage is nil, try aspectMessage for backward compatibilty.
    "
    super list:(self listFromModel) keepSelection:true
!

listFromModel
    "get list from model and return the new list.
     If listMessage is nil, try aspectMessage for backward compatibilty.
    "
    |msg|

    widthOfWidestLine := nil.

    model notNil ifTrue:[
        (msg := listMsg ? aspectMsg) notNil ifTrue:[
            listOfNodes := model perform:msg.

            listOfNodes size ~~ 0 ifTrue:[
                ^ listOfNodes collect:[:n|n name]
            ]
        ]
    ].
    listOfNodes := #().
  ^ #()
! !

!SelectionInTreeView methodsFor:'private queries'!

lengthOfLongestLineBetween:firstLine and:lastLine
    "return the length (in characters) of the longest line in a line-range
    "
    |max|

    max := self widthOfContents:firstLine and:lastLine.
  ^ (max // fontWidth) + 1
!

widthOfContents
    "return the width of the contents in pixels
     - used for scrollbar interface"

    |max|

    list isNil ifTrue:[^ 0].

    (widthOfWidestLine isNil or:[widthOfWidestLine == 0]) ifTrue:[
        widthOfWidestLine := self widthOfContents:1 and:(self size).
    ].
  ^ widthOfWidestLine + (leftMargin * 2)

!

widthOfContents:firstLine and:lastLine
    "return the length (in pixels) of the longest line in a line-range
    "
    |max      "{ Class: SmallInteger }"
     thisLen  "{ Class: SmallInteger }"
     first    "{ Class: SmallInteger }"
     last     "{ Class: SmallInteger }" |

    listOfNodes size == 0 ifTrue:[^ 0].

    last  := listOfNodes size.
    first := firstLine.

    (first > last) ifTrue:[^ 0].

    (lastLine < last) ifTrue:[
        last := lastLine
    ].
    max  := 0.

    listOfNodes from:first to:last do:[:anItem||name|
        thisLen := self xOfStringNode:anItem.

        (name := anItem name) notNil ifTrue:[
            thisLen := thisLen + (name widthOn:self).
        ].
        (thisLen > max) ifTrue:[
            max := thisLen.
        ]
    ].
  ^ max + leftOffset.
!

withoutRedrawAt:anIndex put:aString
    "change a line without redisplay"

    |width|

    width := widthOfWidestLine.
    widthOfWidestLine := nil.

    super withoutRedrawAt:anIndex put:aString.

    (widthOfWidestLine := width) notNil ifTrue:[
        width := self widthOfContents:anIndex and:anIndex.

        width > widthOfWidestLine ifTrue:[
            widthOfWidestLine := width
        ] ifFalse:[
            widthOfWidestLine := nil "/ means: unknown
        ]
    ].


! !

!SelectionInTreeView methodsFor:'queries'!

figureFor:aNode
    "access figure for a node
    "
    aNode hasChildren ifTrue:[
        aNode isExpandable ifTrue:[ ^ imageClosed ]
                          ifFalse:[ ^ imageOpened ]
    ].
  ^ imageItem

    "Modified: 3.7.1997 / 12:31:48 / cg"
!

indexOfNode:aNode
    "returns index of a node
    "
    ^ listOfNodes findFirst:[:n| n == aNode ]
!

xOfFigureNode:aNode
    "origin x where to draw the icon
    "
   ^ ((aNode level-1) * leftIndent) + figuresInset - leftOffset.

!

xOfStringNode:aNode
    "origin x where to draw the text( label )
    "
    ^ (self xOfFigureNode:aNode) + leftIndent

! !

!SelectionInTreeView methodsFor:'selection'!

doubleClicked
    "handle a double click; collapse or expand selected entry
     in case of having children
    "
    |node index|

    (index := self selectedIndex) ~~ 0 ifTrue:[
        node := listOfNodes at:index.

        node hasChildren ifTrue:[
         "/ node has children
            (    doubleClickSelectionBlock isNil
             or:[doubleClickSelectionBlock value:index]) ifFalse:[
                ^ self
            ].
            node isExpandable ifTrue:[node expand]
                             ifFalse:[node collapse].

            node numberOfChildren == 0 ifTrue:[
                "/ no children; redraw selected line (image might change)
                self redrawLine:index.
            ] ifFalse:[
                "/ with children; update list and redraw to end.
                model updateList.
                list := self listFromModel.
                self redrawFromLine:index.
                self contentsChanged.
            ]
        ].
        super doubleClicked.
    ]

!

selectNode:aNode
    "change selection to a node
    "
    self selection:(self indexOfNode:aNode)


!

selectedIndex
    "get single selected index or 0
    "
    selection size == 1 ifTrue:[^ selection first].
    selection isNumber  ifTrue:[^ selection].
  ^ 0
!

selectedNode
    "get the single selected node or nil
    "
    |idx|

    (idx := self selectedIndex) ~~ 0 ifTrue:[
        ^ listOfNodes at:idx
    ].
  ^ nil
! !

!SelectionInTreeView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/Attic/SelTreeV.st,v 1.12 1997-08-07 13:11:45 ca Exp $'
! !
